/****************************************************************************************/
/* Projekt zur Demo (Umgehung der 1k-Grenze)                                            */
/*                                                                                      */
/* Entwickler: Buchgeher Stefan                                                         */
/* Entwicklungsbeginn der Software: 20. Mai 2006                                        */
/* Funktionsfaehig seit: 20. Mai 2006                                                   */
/* Letzte Bearbeitung: 20. Mai 2006                                                     */
/****************************************************************************************/

/****************** Include-Dateien *****************************************************/
#include "PROJEKT.H"
#include <int16CXX.H>                   // Ist fuer die Interrupts notwendig
#include <INLINE.H>                     // Ist fuer Assembleranweisungen notwendig


/****************** Konfigurations-Bits *************************************************/
#pragma config |= 0b.11110100111010


/****************** ISR - Timer0 ********************************************************/

/****************************************************************************************/
/* Interrupt Service Routine (ISR):                                                     */
/****************************************************************************************/
#pragma origin 4

interrupt ISR(void)                     // Interruptroutine
{
    int_save_registers                 // W, STATUS (und PCLATH) retten

    // Beginn der eigentlichen ISR-Routine


    int_restore_registers              // W, STATUS (und PCLATH) Wiederherstellen
}


/****************************************************************************************/
/* INIT:                                                                                */
/*                                                                                      */
/* Aufgabe:                                                                             */
/*   Initialisierung des Prozessor:                                                     */
/****************************************************************************************/
void INIT(void)
{
    // Timer-0-Interrupt
    TMR0 = 0;                          // Timer 0 auf 0 voreinstellen
    OPTION = 0b.1000.0011;             // Pull-Up-Widerstaende am Port B deaktivieren
                                       // Timer-ISR (Vorteiler = 16)

    // Ports
    TRISA = 0xFF;                      // Port A  und Port E muessen bei verwendung als
    TRISE = 0xFF;                      //   ADC als Eingang konfiguriert werden
    TRISB = 0;                         // Port B als Ausgang definieren
    TRISC = 0b.1111.1001;              // Port C: Bits 1 und 2 als Ausgang definieren
                                       // Port C: Bits 0 und 3 bis 7 als Eingang definieren
    TRISD = 0;                         // Port D als Ausgang definieren

    // ADC
    ADON = 1;                          // ADC global einschalten
    ADCON1 = 0b.1000.0010;             // Ganzer Port A als Analog; Vref+ = Vdd, Vref- = Vss
                                       //   und linksbuendig
    ADCS0 = 0;                         // Geschwindigkeit des ADC mit ADCS0 und ADCS1
    ADCS1 = 1;

    // usw.
}


void delay_us(char mikro)
{
	#asm
	wdh9	nop
	nop
	decfsz	mikro,f
	goto	wdh9
	#endasm
}


/****************** Unterprogramm zur Analog-Digital-Wandlung ***************************/

/****************************************************************************************/
/* ADC:                                                                                 */
/*                                                                                      */
/* Aufgabe:                                                                             */
/*   Analog-Digital-Wandlung mit dem PIC-interne ADC fuer den ausgewaehlten ADC-Kanal   */
/*   starten, auf das Wandlungsergebnis warten und dieses dem aufrufenden Programm      */
/*   zurueckgeben.                                                                      */
/*                                                                                      */
/* Uebergabeparameter:                                                                  */
/*   kanal:                                                                             */
/*                                                                                      */
/* Rueckgabeparameter:                                                                  */
/*   6-Bit-Ergebnis der Analog-Digital-Wandlung im EXTERNEN UEBERGABEREGISTER           */
/*   (tmp_uebergabe_uns16)                                                              */
/*                                                                                      */
/* Vorgehensweise:                                                                      */
/*   + ADC-Kanal auswaehlen (Bits <3:5> im Register ADCON0). Dazu muss die Kanal-Nummer */
/*     an diese Position geschoben werden und mit dem Register ADCON0 verknuepft werden */
/*   + ca. 50 Mikrosekunden (us) warten. Diese Zeit benoetigt der PIC um den internen   */
/*     Kondensator mit der zu messenden Analogspannung zu laden.                        */
/*   + Analog-Digital-Wandlung starten. Dazu das Flag GO (im Register ADCON0) setzen.   */
/*   + Warten, bis der PIC mit der Wandlung fertig ist. Der PIC setzt das Flag GO auto- */
/*     matisch zurueck, wenn er mit der Analog-Digital-Wandlung fertig ist.             */
/*   + Aus den Registern ADRESL und ADRESH das Ergebnis zusammensetzen und an das auf-  */
/*     rufende Unterprogramm (oder Hauptprogramm) zurueckgeben                          */
/*                                                                                      */
/* Anmerkung:                                                                           */
/*   Das Ausgabeformat der ADC-Wandlung, also wie die 10 Ergebnisbits in den Registern  */
/*   ADRESL und ADRESH abgelegt werden wird an anderer Stelle (z.B. im Unterprogramm    */
/*   INIT) konfiguriert.                                                                */
/****************************************************************************************/
void ADC(char kanal)
{
    ADCON0 = ADCON0 & 0b.1100.0111;
    kanal = kanal << 3;
    ADCON0 = ADCON0 | kanal;

    delay_us(50);

    GO = 1;
    while(GO);

    tmp_uebergabe_uns16.low8 = ADRESL; // externes Uebergaberegister
    tmp_uebergabe_uns16.high8 = ADRESH;
}


/****************** Hauptprogramm *******************************************************/
void main(void)
{
    char Wert1;

    INIT();                            // Controller initialisieren

    INTCON = 0b.1010.0000;             // Timer0 freigeben durch Setzen von
                                       // GIE und T0IE im Register INTCON
    while(1)
    {
        // aktuelle Werte einlesen
        Soll_Istwerte_einlesen();

        // Test-Routine
        UNTERPROGRAMM2();
        Wert1 = UNTERPROGRAMM3(123);
    }
}
